var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
var cmdm = require('../lib/tfcommand');
var Q = require('q');
var params = require('../lib/parameternames');
var trace = require('../lib/trace');
function describe() {
    return 'get a list of build tasks';
}
exports.describe = describe;
function getArguments() {
    return cmdm.formatArgumentsHint([], [], [params.ALL]);
}
exports.getArguments = getArguments;
function getCommand() {
    // this just offers description for help and to offer sub commands
    return new BuildTaskList;
}
exports.getCommand = getCommand;
// requires auth, connect etc...
exports.isServerOperation = true;
// unless you have a good reason, should not hide
exports.hideBanner = false;
var BuildTaskList = (function (_super) {
    __extends(BuildTaskList, _super);
    function BuildTaskList() {
        _super.apply(this, arguments);
    }
    BuildTaskList.prototype.exec = function (args, options) {
        var _this = this;
        trace("build-tasks-list.exec");
        var deferred = Q.defer();
        var all = options['all'] || false;
        trace("Initializing agent API...");
        var agentapi = this.getWebApi().getTaskAgentApi(this.connection.accountUrl);
        trace("Searching for build tasks...");
        agentapi.getTaskDefinitions(['build'], function (err, statusCode, tasks) {
            if (err) {
                trace("Call to TaskAgentApi.getTaskDefinitions failed with code " + statusCode + ". Message: " + err.message);
                err.statusCode = statusCode;
                deferred.reject(err);
            }
            else {
                trace("Retrieved " + tasks.length + " build tasks from server.");
                if (all) {
                    trace("Listing all build tasks.");
                    deferred.resolve(tasks);
                }
                else {
                    trace("Filtering build tasks to give only the latest versions.");
                    deferred.resolve(_this._getNewestTasks(tasks));
                }
            }
        });
        return deferred.promise;
    };
    BuildTaskList.prototype.output = function (data) {
        if (!data) {
            throw new Error('no tasks supplied');
        }
        if (!(data instanceof Array)) {
            throw new Error('expected an array of tasks');
        }
        data.forEach(function (task) {
            console.log();
            console.log('id   : ' + task.id);
            console.log('name : ' + task.name);
            console.log('friendly name : ' + task.friendlyName);
            console.log('visibility: ' + task.visibility);
            console.log('description: ' + task.description);
            console.log('version: ' + new TaskVersion(task.version).toString());
        });
    };
    /*
     * takes a list of non-unique task definitions and returns only the newest unique definitions
     * TODO: move this code to the server, add a parameter to the controllers
     */
    BuildTaskList.prototype._getNewestTasks = function (allTasks) {
        var taskDictionary = {};
        for (var i = 0; i < allTasks.length; i++) {
            var currTask = allTasks[i];
            if (taskDictionary[currTask.id]) {
                var newVersion = new TaskVersion(currTask.version);
                var knownVersion = new TaskVersion(taskDictionary[currTask.id].version);
                trace("Found additional version of " + currTask.name + " and comparing to the previously encountered version.");
                if (this._compareTaskVersion(newVersion, knownVersion) > 0) {
                    trace("Found newer version of " + currTask.name + ".  Previous: " + knownVersion.toString() + "; New: " + newVersion.toString());
                    taskDictionary[currTask.id] = currTask;
                }
            }
            else {
                trace("Found task " + currTask.name);
                taskDictionary[currTask.id] = currTask;
            }
        }
        var newestTasks = [];
        for (var id in taskDictionary) {
            newestTasks.push(taskDictionary[id]);
        }
        return newestTasks;
    };
    /*
     * compares two versions of tasks, which are stored in version objects with fields 'major', 'minor', and 'patch'
     * @return positive value if version1 > version2, negative value if version2 > version1, 0 otherwise
     */
    BuildTaskList.prototype._compareTaskVersion = function (version1, version2) {
        if (version1.major != version2.major) {
            return version1.major - version2.major;
        }
        if (version1.minor != version2.minor) {
            return version1.minor - version2.minor;
        }
        if (version1.patch != version2.patch) {
            return version1.patch - version2.patch;
        }
        return 0;
    };
    return BuildTaskList;
})(cmdm.TfCommand);
exports.BuildTaskList = BuildTaskList;
var TaskVersion = (function () {
    function TaskVersion(versionData) {
        this.major = versionData.major || 0;
        this.minor = versionData.minor || 0;
        this.patch = versionData.patch || 0;
    }
    TaskVersion.prototype.toString = function () {
        return this.major + "." + this.minor + "." + this.patch;
    };
    return TaskVersion;
})();
