// Copyright (c) Microsoft. All rights reserved.
// Licensed under the MIT license. See LICENSE file in the project root for full license information.
var Q = require('q');
var readline = require("readline"), read = require('read'), async = require('async'), argparser = require('minimist'), trace = require('./trace');
var args = argparser(process.argv.slice(2));
var getValueFromString = function (val, valtype, fallback) {
    var retVal = val;
    switch (valtype) {
        case "number":
            retVal = Number(val);
            if (isNaN(retVal))
                retVal = fallback;
            break;
        case "boolean":
            retVal = ((val.toLowerCase() === "true") || (val === "1"));
            break;
    }
    return retVal;
};
// Q wrapper
function Qget(inputs) {
    var defer = Q.defer();
    this.get(inputs, function (err, result) {
        if (err) {
            defer.reject(err);
            return;
        }
        defer.resolve(result);
    });
    return defer.promise;
}
exports.Qget = Qget;
// done(err, result)
function get(inputs, done) {
    trace('inputs.get');
    var result = {};
    result['_'] = args['_'];
    // TODO: get rid of async and code just with Q
    async.forEachSeries(inputs, function (input, inputDone) {
        if (args[input.arg]) {
            result[input.name] = args[input.arg];
            inputDone(null, null);
            return;
        }
        var msg = 'Enter ' + input.description;
        if (input.def) {
            msg += ' (enter sets ' + input.def + ') ';
        }
        msg += ' > ';
        var silent = input.type === 'password';
        read({ prompt: msg, silent: silent }, function (err, answer) {
            var useVal = answer === "" ? input.def : answer;
            result[input.name] = getValueFromString(useVal, input.type, input.default);
            trace('read: ' + result[input.name]);
            inputDone(null, null);
        });
    }, function (err) {
        if (err) {
            trace('Input reading failed with message: ' + err.message);
            done(err, null);
            return;
        }
        // final validation
        inputs.forEach(function (input) {
            if (input.req && !result[input.name]) {
                trace(input.description + ' is required.');
                done(new Error(input.description + ' is required.'), null);
                return;
            }
        });
        done(null, result);
    });
}
exports.get = get;
